library IsmAvatar;

uses sysutils;

var
 f : array[0..3] of file;
 open : array[0..3] of byte;
 endian : boolean;
 strchar : boolean;
 strmode : byte;

function swap4(l : longint) : longint;
var
 b : array[0..3] of byte;
 t : byte;
begin
 move(l,b,4); //store double in array of bytes
 t := b[0];
 b[0] := b[3];
 b[3] := t;
 t := b[1];
 b[1] := b[2];
 b[2] := t;
 swap4 := longint(b);
end;

function stp(s : string) : pchar;
var
 p : pchar;
begin
 p := stralloc(length(s)+1);
 strpcopy(p,s);
 stp := p;
end;

function _endian(big : real) : real; stdcall;
begin
 if (big <> 0) and (big <> 1) then exit(-1);
 endian := (big = 1);
 _endian := 0;
end;

function _string(mode : real) : real; stdcall;
begin
 if (mode < 0) and (mode > 4) then exit(-1);
 strmode := trunc(mode);
 _string := 0;
end;

function _chars(mode : real) : real; stdcall;
begin
 if (mode <> 1) and (mode <> 2) then exit(-1);
 strchar := (mode = 2);
 _chars := 0;
end;

function _open(fn : pchar; md : real) : real; stdcall;
var m,i : byte;
begin
 if (md <> 0) and (md <> 1) then exit(-1);
 i := trunc(md);
 if not fileexists(fn) and (i <> 1) then exit(-2);
 for m := 0 to 3 do
 begin
  if open[m]>0 then continue;
  assign(f[m],fn);
  {$I-}
  if i = 0 then reset(f[m],1);
  if i = 1 then rewrite(f[m],1);
  {$I+}
  if IOResult <> 0 then exit(-3);
  open[m] := i+1;
  exit(m);
 end;
 _open := -4;
end;

function _close(id : real) : real; stdcall;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]=0 then exit(1);
 close(f[trunc(id)]);
 open[trunc(id)] := 0;
 _close := 0;
end;

function _size(id : real) : real; stdcall;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]=0 then exit(-2);
 {$I-}
 _size := real(filesize(f[trunc(id)]));
 {$I+}
 if IOResult <> 0 then exit(-3);
end;

function _pos(id : real) : real; stdcall;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]=0 then exit(-2);
 {$I-}
 _pos := real(filepos(f[trunc(id)]));
 {$I+}
 if IOResult <> 0 then exit(-3);
end;

function _seek(id,p : real) : real; stdcall;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]=0 then exit(-2);
 {$I-}
 seek(f[trunc(id)],trunc(p));
 {$I+}
 if IOResult <> 0 then exit(-3);
 _seek := 0;
end;

function _read1(id : real) : real; stdcall;
var b : byte;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>1 then exit(-2);
 {$I-}
 blockread(f[trunc(id)],b,1);
 {$I+}
 if IOResult <> 0 then exit(-3);
 _read1 := real(b);
end;

function _read2(id : real) : real; stdcall;
var
 w : word;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>1 then exit(-2);
 {$I-}
 blockread(f[trunc(id)],w,2);
 {$I+}
 if IOResult <> 0 then exit(-3);
 if endian then w := swap(w);
 _read2 := real(w);
end;

function _read3(id : real) : real; stdcall;
var
 l : longint;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>1 then exit(-2);
 {$I-}
 blockread(f[trunc(id)],l,3);
 {$I+}
 if IOResult <> 0 then exit(-3);
 if endian then l := swap(l);
 _read3 := real(l);
end;

function _read4(id : real) : real; stdcall;
var
 l : longint;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>1 then exit(-2);
 {$I-}
 blockread(f[trunc(id)],l,4);
 {$I+}
 if IOResult <> 0 then exit(-3);
 if endian then l := swap4(l);
 _read4 := real(l);
end;

function _readdouble(id : real) : real; stdcall;
var
 d : double;
 b : array[0..7] of byte;
 t,n : byte;
begin
 blockread(f[trunc(id)],d,8);
 if endian then
 begin
  move(d,b,8); //store double in array of bytes
  for n := 0 to 3 do
  begin         //swap bytes
   t := b[n];
   b[n] := b[7-n];
   b[7-n] := t;
  end;
  d := double(b);
 end;
 _readdouble := d;
end;

function _readchars(id,n : real) : pchar; stdcall;
var
 m : cardinal;
 w : word;
// b : byte;
 r : string;
begin
 r := '';
 for m := 1 to trunc(n) do
 begin
  if strchar then
  begin
   blockread(f[trunc(id)],w,2);
   if endian then w := swap(w);
  end
  else
   blockread(f[trunc(id)],w,1);
  r += chr(w);
 end;
 _readchars := stp(r);
end;

function _readstr(id : real) : pchar; stdcall;
var
 l : cardinal;
 b,b2 : byte;
 r : string;
begin
 if strmode > 0 then
 begin
  blockread(f[trunc(id)],l,strmode);
  if endian then l := swap4(l);
  _readstr := _readchars(id,real(l));
 end;
 if strmode = 0 then
 begin
  r := '';
  repeat
   blockread(f[trunc(id)],b,1);
   if strchar then
   begin
    blockread(f[trunc(id)],b2,1);
    if endian then b := b2;
   end;
   if b <> 0 then r += chr(b);
  until b = 0;
  _readstr := stp(r);
 end;
end;

function _write1(id,b : real) : real; stdcall;
var bb : byte;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>2 then exit(-2);
 bb := trunc(b);
 {$I-}
 blockwrite(f[trunc(id)],bb,1);
 {$I+}
 if IOResult <> 0 then exit(-3);
 _write1 := 0;
end;

function _write2(id,w : real) : real; stdcall;
var
 ww : word;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>2 then exit(-2);
 ww := trunc(w);
 if endian then ww := swap(ww);
 {$I-}
 blockwrite(f[trunc(id)],ww,2);
 {$I+}
 if IOResult <> 0 then exit(-3);
 _write2 := 0;
end;

function _write3(id,l : real) : real; stdcall;
var
 ll : longint;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>2 then exit(-2);
 ll := trunc(l);
 if endian then ll := swap(ll);
 {$I-}
 blockwrite(f[trunc(id)],ll,3);
 {$I+}
 if IOResult <> 0 then exit(-3);
 _write3 := 0;
end;

function _write4(id,l : real) : real; stdcall;
var
 ll : longint;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>2 then exit(-2);
 ll := trunc(l);
 if endian then ll := swap4(ll);
 {$I-}
 blockwrite(f[trunc(id)],ll,4);
 {$I+}
 if IOResult <> 0 then exit(-3);
 _write4 := 0;
end;

function _writedouble(id,d : real) : real; stdcall;
var
 dd : double;
 b : array[0..7] of byte;
 t,n : byte;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>2 then exit(-2);
 dd := d;
 if endian then
 begin
  move(dd,b,8); //store double in array of bytes
  for n := 0 to 3 do
  begin         //swap bytes
   t := b[n];
   b[n] := b[7-n];
   b[7-n] := t;
  end;
  dd := double(b);
 end;
 {$I-}
 blockwrite(f[trunc(id)],dd,8);
 {$I+}
 if IOResult <> 0 then exit(-3);
 _writedouble := 0;
end;

function _writechars(id : real; n : pchar) : real; stdcall;
var
 m : longint;
 b : word;
 r : string;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>2 then exit(-2);
 r := strpas(n);
 for m := 1 to length(r) do
 begin
  b := ord(r[m]);
  if strchar and endian then
  begin
   {$I-}
   blockwrite(f[trunc(id)],0,1);
   {$I+}
   if IOResult <> 0 then exit(-3);
  end;
  {$I-}
  blockwrite(f[trunc(id)],b,1);
  {$I+}
  if IOResult <> 0 then exit(-3);
  if strchar and not endian then
  begin
   {$I-}
   blockwrite(f[trunc(id)],0,1);
   {$I+}
   if IOResult <> 0 then exit(-3);
  end;
 end;
 _writechars := 0;
end;

function _writestr(id : real; s : pchar) : real; stdcall;
var
 l : longint;
 r : string;
begin
 if (id > 3) or (id < 0) then exit(-1);
 if open[trunc(id)]<>2 then exit(-2);
 r := strpas(s);
 l := length(r);
 if endian then l := swap4(l);
 if strmode > 0 then
 begin
  {$I-}
  blockwrite(f[trunc(id)],l,strmode);
  {$I+}
  if IOResult <> 0 then exit(-3);
 end;
 _writestr := _writechars(id,s);
 if strmode = 0 then
 begin
  {$I-}
  blockwrite(f[trunc(id)],0,1);
  if strchar then blockwrite(f[trunc(id)],0,1);
  {$I+}
  if IOResult <> 0 then exit(3);
  _writestr := 0;
 end;
end;

function _trans(id1,id2,n : real) : real; stdcall;
var
 l : longint;
 b : byte;
begin
 if (id1 > 3) or (id1 < 0) then exit(-1);
 if open[trunc(id1)]<>1 then exit(-2);
 if (id2 > 3) or (id2 < 0) then exit(-1);
 if open[trunc(id2)]<>2 then exit(-2);
 for l := 1 to trunc(n) do
 begin
  {$I-}
  blockread(f[trunc(id1)],b,1);
  blockwrite(f[trunc(id2)],b,1);
  {$I+}
  if IOResult <> 0 then exit(-3);
 end;
 _trans := 0;
end;

exports _endian,_string,_chars,_open,_close,_size,_pos,_seek,_trans;
exports _read1,_read2,_read3,_read4,_readdouble,_readchars,_readstr;
exports _write1,_write2,_write3,_write4,_writedouble,_writechars,_writestr;
begin
 open[0] := 0;
 open[1] := 0;
 open[2] := 0;
 open[3] := 0;
 endian := false;
 strchar := false;
 strmode := 1;
end.
